/*___________________________________________________________________________________________________

Title:
	display7seg.h v1.0

Description:
	Library for AVR microcontrollers used to control 7 segment displays
	
	For complete details visit
	https://www.programming-electronics-diy.xyz/2021/01/7-segment-display-library-for-avr.html


Author:
 	I. Liviu
	istrateliviu24@yahoo.com

Donate:
	Software development takes time and effort so if you find this useful consider a small donation at:
	paypal.me/alientransducer
_____________________________________________________________________________________________________*/


/* ----------------------------- LICENSE - GNU GPL v3 -----------------------------------------------

* This license must be included in any redistribution.

* Copyright (c) 2021 I. Liviu, www.programming-electronics-diy.xyz (istrateliviu24@yahoo.com)

* Project URL: https://www.programming-electronics-diy.xyz/2021/01/7-segment-display-library-for-avr.html

* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.

* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.

* You should have received a copy of the GNU General Public License
* along with this program. If not, see <https://www.gnu.org/licenses/>.

--------------------------------- END OF LICENSE --------------------------------------------------*/

#ifndef DISPLAY7SEG_H_
#define DISPLAY7SEG_H_

/*************************************************************
	INCLUDES
**************************************************************/
#include <inttypes.h>
#include <avr/io.h>
#include <util/atomic.h>

/*************************************************************
	SYSTEM SETTINGS
**************************************************************/
#ifndef true
	#define true							1
	#define false							0
#endif

#define TRUE								1
#define FALSE								0

#define NC									-1 // not connected
#define DISPLAY_7SEG_COMMON_CATHODE			0
#define DISPLAY_7SEG_COMMON_ANODE			1

/*************************************************************
	USER SETUP SECTION
**************************************************************/
// General display characteristics
#define DISPLAY_7SEG_NR_OF_DIGITS			4 // how many digits the display has
#define DISPLAY_7SEG_TYPE					DISPLAY_7SEG_COMMON_CATHODE

// Display pin connections
#define DISPLAY_7SEG_PORT					PORTD // PORTx - the port where the segments are connected

// IMPORTANT. All pins must be on the same port. 
// The numbers assigned are just for guidance. Replace them according to your circuit.
#define SEGMENT_A_PIN						7
#define SEGMENT_B_PIN						5
#define SEGMENT_C_PIN						1
#define SEGMENT_D_PIN						3
#define SEGMENT_E_PIN						4
#define SEGMENT_F_PIN						6
#define SEGMENT_G_PIN						0

// The dot can be on a different port
#define SEGMENT_DOT_PORT					PORTD
#define SEGMENT_DOT_PIN						PD2

// Common anode or common cathode pin for each digit
#define DISPLAY_DIGIT_1_DDR					DDRC
#define DISPLAY_DIGIT_1_PORT				PORTC
#define DISPLAY_DIGIT_1_PIN					PC0

#define DISPLAY_DIGIT_2_DDR					DDRC
#define DISPLAY_DIGIT_2_PORT				PORTC
#define DISPLAY_DIGIT_2_PIN					PC1

#define DISPLAY_DIGIT_3_DDR					DDRC
#define DISPLAY_DIGIT_3_PORT				PORTC
#define DISPLAY_DIGIT_3_PIN					PC2

#define DISPLAY_DIGIT_4_DDR					DDRC
#define DISPLAY_DIGIT_4_PORT				PORTC
#define DISPLAY_DIGIT_4_PIN					PC3

#define DISPLAY_DIGIT_5_DDR					NC
#define DISPLAY_DIGIT_5_PORT				NC
#define DISPLAY_DIGIT_5_PIN					NC

#define DISPLAY_DIGIT_6_DDR					NC
#define DISPLAY_DIGIT_6_PORT				NC
#define DISPLAY_DIGIT_6_PIN					NC

// To be able to use display7seg_printFloat() set this to TRUE. If you are not planning to
// display floats then set this to FALSE to save space. 
// If you are using floats, then uncomment MATH_LIB = -lm
// in your Make file to reduce the loaded math library to half. 
#define LCD_DISPLAY_FLOATS 					FALSE // TRUE or FALSE

/*************************************************************
	--END OF USER SETUP SECTION
**************************************************************/

/*************************************************************
	SYSTEM SETTINGS
**************************************************************/
#define DISPLAY_7SEG_DDR					*(&DISPLAY_7SEG_PORT - 1)
#define SEGMENT_DOT_DDR						*(&SEGMENT_DOT_PORT - 1)

// Digits 0-9
#define DISPLAY_7SEG_DIGIT_0	((1 << SEGMENT_A_PIN) | (1 << SEGMENT_B_PIN) | (1 << SEGMENT_C_PIN) | (1 << SEGMENT_D_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_F_PIN))
#define DISPLAY_7SEG_DIGIT_1	((1 << SEGMENT_B_PIN) | (1 << SEGMENT_C_PIN))
#define DISPLAY_7SEG_DIGIT_2	((1 << SEGMENT_A_PIN) | (1 << SEGMENT_B_PIN) | (1 << SEGMENT_G_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_D_PIN))
#define DISPLAY_7SEG_DIGIT_3	((1 << SEGMENT_A_PIN) | (1 << SEGMENT_B_PIN) | (1 << SEGMENT_G_PIN) | (1 << SEGMENT_C_PIN) | (1 << SEGMENT_D_PIN))
#define DISPLAY_7SEG_DIGIT_4	((1 << SEGMENT_F_PIN) | (1 << SEGMENT_G_PIN) | (1 << SEGMENT_B_PIN) | (1 << SEGMENT_C_PIN))
#define DISPLAY_7SEG_DIGIT_5	((1 << SEGMENT_A_PIN) | (1 << SEGMENT_F_PIN) | (1 << SEGMENT_G_PIN) | (1 << SEGMENT_C_PIN) | (1 << SEGMENT_D_PIN))
#define DISPLAY_7SEG_DIGIT_6	((1 << SEGMENT_A_PIN) | (1 << SEGMENT_F_PIN) | (1 << SEGMENT_G_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_D_PIN) | (1 << SEGMENT_C_PIN))
#define DISPLAY_7SEG_DIGIT_7	((1 << SEGMENT_A_PIN) | (1 << SEGMENT_B_PIN) | (1 << SEGMENT_C_PIN))
#define DISPLAY_7SEG_DIGIT_8	((1 << SEGMENT_A_PIN) | (1 << SEGMENT_B_PIN) | (1 << SEGMENT_C_PIN) | (1 << SEGMENT_D_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_F_PIN) | (1 << SEGMENT_G_PIN))
#define DISPLAY_7SEG_DIGIT_9	((1 << SEGMENT_A_PIN) | (1 << SEGMENT_B_PIN) | (1 << SEGMENT_C_PIN) | (1 << SEGMENT_D_PIN) | (1 << SEGMENT_F_PIN) | (1 << SEGMENT_G_PIN))

// Minus "-" sign
#define DISPLAY_7SEG_DIGIT_MINUS	(1 << SEGMENT_G_PIN)

// Space
#define DISPLAY_7SEG_DIGIT_SPACE	(0x00)

// Letters
#define DISPLAY_7SEG_DIGIT_a	((1 << SEGMENT_A_PIN) | (1 << SEGMENT_B_PIN) | (1 << SEGMENT_C_PIN) | (1 << SEGMENT_D_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_G_PIN))
#define DISPLAY_7SEG_DIGIT_b	((1 << SEGMENT_C_PIN) | (1 << SEGMENT_D_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_F_PIN) | (1 << SEGMENT_G_PIN))
#define DISPLAY_7SEG_DIGIT_c	((1 << SEGMENT_G_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_D_PIN))
#define DISPLAY_7SEG_DIGIT_d	((1 << SEGMENT_B_PIN) | (1 << SEGMENT_C_PIN) | (1 << SEGMENT_D_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_G_PIN))
#define DISPLAY_7SEG_DIGIT_o	((1 << SEGMENT_G_PIN) | (1 << SEGMENT_C_PIN) | (1 << SEGMENT_D_PIN) | (1 << SEGMENT_E_PIN))
#define DISPLAY_7SEG_DIGIT_r	((1 << SEGMENT_E_PIN) | (1 << SEGMENT_G_PIN))
#define DISPLAY_7SEG_DIGIT_t	((1 << SEGMENT_F_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_G_PIN) | (1 << SEGMENT_D_PIN))

#define DISPLAY_7SEG_DIGIT_A	((1 << SEGMENT_A_PIN) | (1 << SEGMENT_B_PIN) | (1 << SEGMENT_C_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_F_PIN) | (1 << SEGMENT_G_PIN))
#define DISPLAY_7SEG_DIGIT_C	((1 << SEGMENT_A_PIN) | (1 << SEGMENT_F_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_D_PIN))
#define DISPLAY_7SEG_DIGIT_E	((1 << SEGMENT_A_PIN) | (1 << SEGMENT_F_PIN) | (1 << SEGMENT_G_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_D_PIN))
#define DISPLAY_7SEG_DIGIT_F	((1 << SEGMENT_A_PIN) | (1 << SEGMENT_F_PIN) | (1 << SEGMENT_G_PIN) | (1 << SEGMENT_E_PIN))
#define DISPLAY_7SEG_DIGIT_G	DISPLAY_7SEG_DIGIT_6
#define DISPLAY_7SEG_DIGIT_H	((1 << SEGMENT_F_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_G_PIN) | (1 << SEGMENT_B_PIN) | (1 << SEGMENT_C_PIN))
#define DISPLAY_7SEG_DIGIT_I	((1 << SEGMENT_F_PIN) | (1 << SEGMENT_E_PIN))
#define DISPLAY_7SEG_DIGIT_J	((1 << SEGMENT_B_PIN) | (1 << SEGMENT_C_PIN) | (1 << SEGMENT_D_PIN))
#define DISPLAY_7SEG_DIGIT_L	((1 << SEGMENT_F_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_D_PIN))
#define DISPLAY_7SEG_DIGIT_N	((1 << SEGMENT_A_PIN) | (1 << SEGMENT_B_PIN) | (1 << SEGMENT_C_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_F_PIN))
#define DISPLAY_7SEG_DIGIT_O	DISPLAY_7SEG_DIGIT_0
#define DISPLAY_7SEG_DIGIT_P	((1 << SEGMENT_A_PIN) | (1 << SEGMENT_B_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_F_PIN) | (1 << SEGMENT_G_PIN))
#define DISPLAY_7SEG_DIGIT_Q	((1 << SEGMENT_A_PIN) | (1 << SEGMENT_B_PIN) | (1 << SEGMENT_C_PIN) | (1 << SEGMENT_F_PIN) | (1 << SEGMENT_G_PIN))
#define DISPLAY_7SEG_DIGIT_S	((1 << SEGMENT_A_PIN) | (1 << SEGMENT_F_PIN) | (1 << SEGMENT_G_PIN) | (1 << SEGMENT_C_PIN) | (1 << SEGMENT_D_PIN))
#define DISPLAY_7SEG_DIGIT_U	((1 << SEGMENT_B_PIN) | (1 << SEGMENT_C_PIN) | (1 << SEGMENT_D_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_F_PIN))
#define DISPLAY_7SEG_DIGIT_Y	((1 << SEGMENT_B_PIN) | (1 << SEGMENT_C_PIN) | (1 << SEGMENT_D_PIN) | (1 << SEGMENT_F_PIN) | (1 << SEGMENT_G_PIN))

#define DISPLAY_7SEG_ALL_PINS_MASK	((1 << SEGMENT_A_PIN) | (1 << SEGMENT_B_PIN) | (1 << SEGMENT_C_PIN) | (1 << SEGMENT_D_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_F_PIN) | (1 << SEGMENT_G_PIN))

//=============================================================================
// Global VARIABLES
//=============================================================================
typedef uint8_t bool;

uint8_t const DISPLAY_7SEG_FONTS[] = {
   DISPLAY_7SEG_DIGIT_0,
   DISPLAY_7SEG_DIGIT_1,
   DISPLAY_7SEG_DIGIT_2,
   DISPLAY_7SEG_DIGIT_3,
   DISPLAY_7SEG_DIGIT_4,
   DISPLAY_7SEG_DIGIT_5,
   DISPLAY_7SEG_DIGIT_6,
   DISPLAY_7SEG_DIGIT_7,
   DISPLAY_7SEG_DIGIT_8,
   DISPLAY_7SEG_DIGIT_9
};

static uint8_t DISPLAY_7SEG_BUFFER[DISPLAY_7SEG_NR_OF_DIGITS] = {0};
static volatile uint8_t DISPLAY_7SEG_DOT_STATES = 0;
static volatile uint8_t DISPLAY_7SEG_ACTIVE_DIGIT = 0;

//=============================================================================
// Function PROTOTYPES
//=============================================================================
void display7seg_init(void);
void display7seg_printInt(int32_t number, int8_t start_position, int8_t nrOfDigits);
void display7seg_printChar(int8_t start_position, uint8_t character);

#if LCD_DISPLAY_FLOATS == TRUE
	void display7seg_printFloat(float float_number, int8_t start_position, int8_t nrOfDigits, int8_t nrOfDecimals);
#endif

void display7seg_clearDisplay(void);
void display7seg_dotOn(uint8_t dot_number);
void display7seg_dotOff(uint8_t dot_number);
void display7seg_dotAnimation(void);
void display7seg_multiplexer(void);

void display7seg_turnDigitsOff(void);


//=============================================================================
//	FUNCTIONS
//=============================================================================

void display7seg_printInt(int32_t number, int8_t start_position, int8_t nrOfDigits){
	bool isNegative = false;
	uint8_t length = 0;
	uint8_t divide = 0;
	uint8_t idx = 0; // bust be unsigned
	int32_t number_copy = number;
	

	// Do not return here in case padding is necessary
	if(number == 0) length = 1;

	// Find the number of digits
	while(number_copy != 0){
		length++;
		number_copy /= 10;
	}

	// Restore the number
	number_copy = number;
	
	// Check if padding with 0 is necessary
	nrOfDigits -= length;
	if(nrOfDigits < 0) nrOfDigits = 0;
	
	// Check if number is negative and convert it to a positive
	if(number < 0){
		isNegative = true;
		number_copy = 0 - number_copy;
		
		// Add 1 for the minus sign
		idx++;
	}

	idx += length + start_position + nrOfDigits - 2;
	divide = length + nrOfDigits;
	
	// Convert int to string and put in the array
	while(divide){
		// Trim the length of the number if it's bigger than the limit
		if(idx < DISPLAY_7SEG_NR_OF_DIGITS){
			DISPLAY_7SEG_BUFFER[idx] = DISPLAY_7SEG_FONTS[number_copy % 10];
		}
		
		number_copy /= 10;
		idx--;
		divide--;
	}

	if(isNegative && idx < DISPLAY_7SEG_NR_OF_DIGITS) DISPLAY_7SEG_BUFFER[idx] = DISPLAY_7SEG_DIGIT_MINUS;
}



void display7seg_printChar(int8_t start_position, uint8_t character){
	if(start_position < DISPLAY_7SEG_NR_OF_DIGITS + 1 && start_position > 0){
		DISPLAY_7SEG_BUFFER[start_position - 1] = character;
	}
}



#if LCD_DISPLAY_FLOATS == TRUE
	void display7seg_printFloat(float float_number, int8_t start_position, int8_t nrOfDigits, int8_t nrOfDecimals){
	   
		int32_t integer_part_1 = float_number;
		int32_t integer_part_2 = 0;
		float fractional_part = float_number - integer_part_1;
		
		uint8_t float_length = nrOfDecimals;
		uint8_t integer_part_1_length = 0;
		int8_t integer_part_2_pos_offset = 0;

		if(nrOfDecimals == 0) float_length = 1;

		// Trim the decimals as a function of nrOfDecimals
		while(float_length){
			fractional_part *= 10;
			float_length--;
		}

		integer_part_2 = fractional_part;

		// Find the number of digits
		while(integer_part_1 != 0){
			integer_part_1_length++;
			integer_part_1 /= 10;
		}

		// Restore the number
		integer_part_1 = float_number;

		// If 0 make length 1 since the while loop was skipped
		if(integer_part_1 == 0) integer_part_1_length++;

		// Convert fractional part to a positive 
		if(integer_part_2 < 0) integer_part_2 = 0 - integer_part_2;

		// The case where the float is -0.xxxx
		if(float_number < 0){
		
			// int function will not get -0 but 0 so the minus sign is displayed here
			if(integer_part_1 == 0){
				DISPLAY_7SEG_BUFFER[start_position - 1] = DISPLAY_7SEG_DIGIT_MINUS;

				// Add 1 for the minus sign
				start_position++;
			}else{
				integer_part_2_pos_offset++;
			}

		}

		// Display integer part 1
		display7seg_printInt(integer_part_1, start_position, nrOfDigits);

		nrOfDigits -= integer_part_1_length;
		if(nrOfDigits < 0) nrOfDigits = 0;
		integer_part_2_pos_offset += start_position + integer_part_1_length + nrOfDigits;

		// Display the dot
		display7seg_dotOn(integer_part_2_pos_offset - 1);

		// Display integer part 2
		display7seg_printInt(integer_part_2, integer_part_2_pos_offset, nrOfDecimals);
	}
	
#endif



void display7seg_clearDisplay(void){
	uint8_t i = DISPLAY_7SEG_NR_OF_DIGITS;
	
	// Sometimes the display is not cleared without this
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE){
		while(i){
			i--;
			DISPLAY_7SEG_BUFFER[i] = 0;
		}
	}
	
	DISPLAY_7SEG_DOT_STATES = 0;
}



void display7seg_dotOn(uint8_t dot_number){
	// DISPLAY_7SEG_ACTIVE_DIGIT starts from 0 but dot_number starts from 1 so 1 is subtracted
	DISPLAY_7SEG_DOT_STATES |= 1 << (dot_number - 1);
}




void display7seg_dotOff(uint8_t dot_number){
	DISPLAY_7SEG_DOT_STATES &= ~(1 << (dot_number - 1));
}



void display7seg_dotAnimation(void){
   uint8_t i = 0;
   
   for(i=1; i<DISPLAY_7SEG_NR_OF_DIGITS + 1; i++){
       display7seg_dotOn(i);
       _delay_ms(100);
    }

    for(i=1; i<DISPLAY_7SEG_NR_OF_DIGITS + 1; i++){
       display7seg_dotOff(i);
       _delay_ms(100);
    }
}



void display7seg_multiplexer(void){
	// 106us @ 1MHz CPU
	// 10us @ 8MHz CPU
	
	display7seg_turnDigitsOff();
	
	#if DISPLAY_7SEG_TYPE == DISPLAY_7SEG_COMMON_CATHODE
   
		// Display the active segments for the current digit
		DISPLAY_7SEG_PORT = DISPLAY_7SEG_BUFFER[DISPLAY_7SEG_ACTIVE_DIGIT];

		// Turn on or off the dot for the active digit
		if(DISPLAY_7SEG_DOT_STATES & (1 << DISPLAY_7SEG_ACTIVE_DIGIT)){
			SEGMENT_DOT_PORT |= (1 << SEGMENT_DOT_PIN);
		}
		
		#if DISPLAY_7SEG_NR_OF_DIGITS > 0
			switch(DISPLAY_7SEG_ACTIVE_DIGIT){
				case 0:
					DISPLAY_DIGIT_1_PORT |= (1 << DISPLAY_DIGIT_1_PIN);
				break;

			#if DISPLAY_7SEG_NR_OF_DIGITS > 1
				case 1:
					DISPLAY_DIGIT_2_PORT |= (1 << DISPLAY_DIGIT_2_PIN);
				break;
			#endif

			#if DISPLAY_7SEG_NR_OF_DIGITS > 2
				case 2:
					DISPLAY_DIGIT_3_PORT |= (1 << DISPLAY_DIGIT_3_PIN);
				break;
			#endif

			#if DISPLAY_7SEG_NR_OF_DIGITS > 3
				case 3:
					DISPLAY_DIGIT_4_PORT |= (1 << DISPLAY_DIGIT_4_PIN);
				break;
			#endif

			#if DISPLAY_7SEG_NR_OF_DIGITS > 4
				case 4:
					DISPLAY_DIGIT_5_PORT |= (1 << DISPLAY_DIGIT_5_PIN);
				break;
			#endif

			#if DISPLAY_7SEG_NR_OF_DIGITS > 5
				case 5:
					DISPLAY_DIGIT_6_PORT |= (1 << DISPLAY_DIGIT_6_PIN);
				break;
			#endif
			}
		#endif
		
		
	#elif DISPLAY_7SEG_TYPE == DISPLAY_7SEG_COMMON_ANODE
		// Display the active segments for the current digit
		DISPLAY_7SEG_PORT = ~(DISPLAY_7SEG_BUFFER[DISPLAY_7SEG_ACTIVE_DIGIT]);

		// Turn on or off the dot for the active digit
		if(DISPLAY_7SEG_DOT_STATES & (1 << DISPLAY_7SEG_ACTIVE_DIGIT)){
			SEGMENT_DOT_PORT &= ~(1 << SEGMENT_DOT_PIN);
		}
		
		#if DISPLAY_7SEG_NR_OF_DIGITS > 0
			switch(DISPLAY_7SEG_ACTIVE_DIGIT){
				case 0:
					DISPLAY_DIGIT_1_PORT &= ~(1 << DISPLAY_DIGIT_1_PIN);
				break;

			#if DISPLAY_7SEG_NR_OF_DIGITS > 1
				case 1:
					DISPLAY_DIGIT_2_PORT &= ~(1 << DISPLAY_DIGIT_2_PIN);
				break;
			#endif

			#if DISPLAY_7SEG_NR_OF_DIGITS > 2
				case 2:
					DISPLAY_DIGIT_3_PORT &= ~(1 << DISPLAY_DIGIT_3_PIN);
				break;
			#endif

			#if DISPLAY_7SEG_NR_OF_DIGITS > 3
				case 3:
					DISPLAY_DIGIT_4_PORT &= ~(1 << DISPLAY_DIGIT_4_PIN);
				break;
			#endif

			#if DISPLAY_7SEG_NR_OF_DIGITS > 4
				case 4:
					DISPLAY_DIGIT_5_PORT &= ~(1 << DISPLAY_DIGIT_5_PIN);
				break;
			#endif

			#if DISPLAY_7SEG_NR_OF_DIGITS > 5
				case 5:
					DISPLAY_DIGIT_6_PORT &= ~(1 << DISPLAY_DIGIT_6_PIN);
				break;
			#endif
			}
		#endif
		
	#endif
		
       
	DISPLAY_7SEG_ACTIVE_DIGIT++;
	if(DISPLAY_7SEG_ACTIVE_DIGIT > DISPLAY_7SEG_NR_OF_DIGITS - 1) DISPLAY_7SEG_ACTIVE_DIGIT = 0;
}




void display7seg_init(void){
	#if DISPLAY_7SEG_TYPE == DISPLAY_7SEG_COMMON_CATHODE
		// Set pins low
		DISPLAY_7SEG_PORT &= ~((1 << SEGMENT_A_PIN) | (1 << SEGMENT_B_PIN) | (1 << SEGMENT_C_PIN) | (1 << SEGMENT_D_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_F_PIN) | (1 << SEGMENT_G_PIN));
		SEGMENT_DOT_PORT &= ~(1 << SEGMENT_DOT_PIN);
		
	#elif DISPLAY_7SEG_TYPE == DISPLAY_7SEG_COMMON_ANODE
		// Set pins high
		DISPLAY_7SEG_PORT |= ((1 << SEGMENT_A_PIN) | (1 << SEGMENT_B_PIN) | (1 << SEGMENT_C_PIN) | (1 << SEGMENT_D_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_F_PIN) | (1 << SEGMENT_G_PIN));
		SEGMENT_DOT_PORT |= (1 << SEGMENT_DOT_PIN);
		
	#endif


	// Set pins to output
	DISPLAY_7SEG_DDR |= (1 << SEGMENT_A_PIN) | (1 << SEGMENT_B_PIN) | (1 << SEGMENT_C_PIN) | (1 << SEGMENT_D_PIN) | (1 << SEGMENT_E_PIN) | (1 << SEGMENT_F_PIN) | (1 << SEGMENT_G_PIN);
	SEGMENT_DOT_DDR |= 1 << SEGMENT_DOT_PIN;

	#if DISPLAY_7SEG_NR_OF_DIGITS > 0
	DISPLAY_DIGIT_1_DDR |= 1 << DISPLAY_DIGIT_1_PIN;
	#endif

	#if DISPLAY_7SEG_NR_OF_DIGITS > 1
	DISPLAY_DIGIT_2_DDR |= 1 << DISPLAY_DIGIT_2_PIN;
	#endif

	#if DISPLAY_7SEG_NR_OF_DIGITS > 2
	DISPLAY_DIGIT_3_DDR |= 1 << DISPLAY_DIGIT_3_PIN;
	#endif

	#if DISPLAY_7SEG_NR_OF_DIGITS > 3
	DISPLAY_DIGIT_4_DDR |= 1 << DISPLAY_DIGIT_4_PIN;
	#endif

	#if DISPLAY_7SEG_NR_OF_DIGITS > 4
	DISPLAY_DIGIT_5_DDR |= 1 << DISPLAY_DIGIT_5_PIN;
	#endif

	#if DISPLAY_7SEG_NR_OF_DIGITS > 5
	DISPLAY_DIGIT_6_DDR |= 1 << DISPLAY_DIGIT_6_PIN;
	#endif


	display7seg_turnDigitsOff();
}
 
 
void display7seg_turnDigitsOff(void){

#if DISPLAY_7SEG_TYPE == DISPLAY_7SEG_COMMON_CATHODE

    #if DISPLAY_7SEG_NR_OF_DIGITS > 0
    DISPLAY_DIGIT_1_PORT &= ~(1 << DISPLAY_DIGIT_1_PIN);
    #endif
    
    #if DISPLAY_7SEG_NR_OF_DIGITS > 1
    DISPLAY_DIGIT_2_PORT &= ~(1 << DISPLAY_DIGIT_2_PIN);
    #endif
    
    #if DISPLAY_7SEG_NR_OF_DIGITS > 2
    DISPLAY_DIGIT_3_PORT &= ~(1 << DISPLAY_DIGIT_3_PIN);
    #endif
    
    #if DISPLAY_7SEG_NR_OF_DIGITS > 3
    DISPLAY_DIGIT_4_PORT &= ~(1 << DISPLAY_DIGIT_4_PIN);
    #endif
    
    #if DISPLAY_7SEG_NR_OF_DIGITS > 4
    DISPLAY_DIGIT_5_PORT &= ~(1 << DISPLAY_DIGIT_5_PIN);
    #endif
    
    #if DISPLAY_7SEG_NR_OF_DIGITS > 5
    DISPLAY_DIGIT_6_PORT &= ~(1 << DISPLAY_DIGIT_6_PIN);
    #endif
	
#elif DISPLAY_7SEG_TYPE == DISPLAY_7SEG_COMMON_ANODE

	#if DISPLAY_7SEG_NR_OF_DIGITS > 0
    DISPLAY_DIGIT_1_PORT |= (1 << DISPLAY_DIGIT_1_PIN);
    #endif
    
    #if DISPLAY_7SEG_NR_OF_DIGITS > 1
    DISPLAY_DIGIT_2_PORT |= (1 << DISPLAY_DIGIT_2_PIN);
    #endif
    
    #if DISPLAY_7SEG_NR_OF_DIGITS > 2
    DISPLAY_DIGIT_3_PORT |= (1 << DISPLAY_DIGIT_3_PIN);
    #endif
    
    #if DISPLAY_7SEG_NR_OF_DIGITS > 3
    DISPLAY_DIGIT_4_PORT |= (1 << DISPLAY_DIGIT_4_PIN);
    #endif
    
    #if DISPLAY_7SEG_NR_OF_DIGITS > 4
    DISPLAY_DIGIT_5_PORT |= (1 << DISPLAY_DIGIT_5_PIN);
    #endif
    
    #if DISPLAY_7SEG_NR_OF_DIGITS > 5
    DISPLAY_DIGIT_6_PORT |= (1 << DISPLAY_DIGIT_6_PIN);
    #endif
#endif

}

#endif // DISPLAY7SEG_H_
